hex_color_dict = {
    "red": "FF0000",
    "green": "00FF00",
    "blue": "0000FF",
    "yellow": "FFFF00",
    "purple": "800080",
    "cyan": "00FFFF",
    "magenta": "FF00FF",
    "lime": "00FF00",
    "pink": "FFC0CB",
    "teal": "008080",
    "lavender": "E6E6FA",
    "brown": "A52A2A",
    "beige": "F5F5DC",
    "maroon": "800000",
    "mint": "98FF98",
    "olive": "808000",
    "coral": "FF7F50",
    "navy": "000080",
    "grey": "808080",
    "white": "FFFFFF",
    "black": "000000",
    "orange": "FFA500",
    "indigo": "4B0082",
    "violet": "EE82EE",
    "gold": "FFD700",
    "silver": "C0C0C0",
    "rose": "FF007F",
    "emerald": "50C878",
    "ruby": "E0115F",
    "sapphire": "0F52BA",
    "amber": "FFBF00",
    "crimson": "DC143C",
    "darkgreen": "006400",
    "darkblue": "00008B",
    "darkred": "8B0000",
    "lightgreen": "90EE90",
    "lightblue": "ADD8E6",
    "lightyellow": "FFFFE0",
    "lightpink": "FFB6C1",
    "darkpurple": "800080",
    "lightpurple": "D8BFD8",
    "turquoise": "40E0D0",
    "tan": "D2B48C",
    "lightgrey": "D3D3D3",
    "darkgrey": "A9A9A9",
    "ivory": "FFFFF0",
    "khaki": "F0E68C",
    "lavenderblush": "FFF0F5",
    "lightcoral": "F08080",
    "salmon": "FA8072",
    "darksalmon": "E9967A",
    "lightsalmon": "FFA07A",
    "darkorange": "FF8C00",
    "linen": "FAF0E6",
    "peachpuff": "FFDAB9",
    "seashell": "FFF5EE",
    "snow": "FFFAFA",
    "wheat": "F5DEB3",
    "mistyrose": "FFE4E1",
    "azure": "F0FFFF",
    "aliceblue": "F0F8FF",
    "honeydew": "F0FFF0",
    "mintcream": "F5FFFA"
}