from list_sets.ticker_lists import most_active_tickers

from apis.webull.webull_trading import WebullTrading



class ReferenceMarket(WebullTrading):
    def __init__(self):
        self.webull_headers = self.headers





    
